/*
   Notice that saving results in a List does NOT work
   because a List is not "thread safe". When multiple
   threads try to insert an item into the List at the
   same time, we can get a race condition and one of
   the items may get "lost".
*/

import java.util.concurrent.Executors;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.TimeUnit;

import java.util.List;
import java.util.ArrayList;

/**
   Execute 1,000 tasks by placing the tasks in a "task pool" that
   is worked on by a fixed number of threads from a "thread pool".
*/
public class ThreadPoolExample_saved_results_v1
{
   public static void main(String[] args)
   {
      final List<List<Long>> results = new ArrayList<>();

      final long startTime = System.currentTimeMillis();

      for (long i = 0; i < 1_000; ++i) // try 10_000
      {
       //final long x = i + 50_000_000_000_000_001L;
         final long x = i + 50_001L;
         // submit a Runnable that adds its result to the List
         executor.execute( ()->results.add(Primes.factorize(x)) );
      }
      executor.shutdown();

      try{executor.awaitTermination(1, TimeUnit.MINUTES);}catch(InterruptedException e){}

      for (List<Long> result : results)
      {
          System.out.println( result );
      }

      final long stopTime = System.currentTimeMillis();
      System.out.println("(" + (stopTime - startTime) + "ms)");
      System.out.println("length of results = " + results.size() );
   }

   private static final int cores = Runtime.getRuntime().availableProcessors();
   private static final ExecutorService executor = Executors.newFixedThreadPool(cores);
}
