/*
   Instead of saving each completed result in a List,
   this example stores unfinished Future results in a
   List. After all the work has been submitted to the
   threadpool, we wait on each Future object to complete.

   Notice how the single main() thread is putting items
   in the (non thread safe) List.
*/

import java.util.concurrent.Executors;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.ExecutionException;

import java.util.List;
import java.util.ArrayList;

/**
   Execute 1,000 tasks by placing the tasks in a "task pool" that
   is worked on by a fixed number of threads from a "thread pool".
*/
public class ThreadPoolExample_saved_results_v2
{
   public static void main(String[] args)
   {
      final List<Future<List<Long>>> results = new ArrayList<>();

      final long startTime = System.currentTimeMillis();

      for (long i = 0; i < 1_000; ++i) // try 10_000
      {
         final long x = i + 50_000_000_000_000_001L;
         // submit a Callable, get back a Future that's added to the List
         results.add( executor.submit( ()->Primes.factorize(x)) );
      }
      executor.shutdown();

      for (Future<List<Long>> result : results)
      {
         try
         {
            System.out.println( result.get() ); // this waits for each result to be ready
         }
         catch(InterruptedException e){}
         catch(ExecutionException e){}
      }

      final long stopTime = System.currentTimeMillis();
      System.out.println("(" + (stopTime - startTime) + "ms)");
      System.out.println("length of results = " + results.size() );
   }

   private static final int cores = Runtime.getRuntime().availableProcessors();
   private static final ExecutorService executor = Executors.newFixedThreadPool(cores);
}
